#@+leo-ver=4
#@+node:@file C:/Projects/PyMotion/elemhandle.py
#@@language python
#@<< elemhandle declarations >>
#@+node:<< elemhandle declarations >>
#!/usr/bin/env python

import math

#@-node:<< elemhandle declarations >>
#@nl
#@+others
#@+node:normalize(my_vector)
def normalize(my_vector):
    "normalize a vector of any dimension"
    magnitude = 0
    for i in range(len(my_vector)):
        magnitude += my_vector[i] * my_vector[i]
    magnitude = math.sqrt(magnitude)

    norm_vector = my_vector
    for i in range(len(my_vector)):
        norm_vector[i] = my_vector[i] / magnitude
        

    return norm_vector
#@-node:normalize(my_vector)
#@+node:magnitude(my_vector)
def magnitude(my_vector):
    "find the magnitude of a vector of any dimension"
    magnitude = 0
    for i in range(len(my_vector)):
        magnitude += my_vector[i] * my_vector[i]
    magnitude = math.sqrt(magnitude)

    return magnitude
#@-node:magnitude(my_vector)
#@+node:create_point(pos, num_elems)
def create_point(pos, num_elems):
    "creates several elements at one point"
    elems = []

    for i in range(num_elems):
        elems.append(pos)

    return elems
#@-node:create_point(pos, num_elems)
#@+node:create_line(start, dir, length, num_elems)
def create_line(start, dir, length, num_elems):
    "creates a straight line of evenly distributed elements"
    dir_norm = normalize(dir)
    elems = []

    for i in range(num_elems):
        dist = float(i) / float(num_elems) * length
        elem_x = dist * dir_norm[0] + start[0]
        elem_y = dist * dir_norm[1] + start[1]

        elems.append([int(elem_x), int(elem_y)])

    return elems
#@-node:create_line(start, dir, length, num_elems)
#@+node:create_circle(center, radius, num_elems)
def create_circle(center, radius, num_elems):
    "creates a circle of evenly distributed elements"
    elems = []
    for i in range(num_elems):
        angle = float(i) / float(num_elems) * (2 * math.pi)
        elem_x = radius * math.cos(angle) + center[0]
        elem_y = radius * math.sin(angle) + center[1]

        elems.append([int(elem_x), int(elem_y)])

    return elems
#@-node:create_circle(center, radius, num_elems)
#@+node:circ_motion(center, core_elements, rate, rd, td, tick)
def circ_motion(center, core_elements, rate, rd, td, tick):   
    "differential motion in a circle"
    num_elems = len(core_elements)
    
    elems = []
    for x in range(num_elems):
        vec_from_center = [core_elements[x][0] - center[0],
                           core_elements[x][1] - center[1]]
        angle_from_center = math.atan2(vec_from_center[1], vec_from_center[0])
        
        diff_angle = (num_elems - x) * (rate * tick) * td
        diff_angle += angle_from_center

        diff_mag = magnitude(vec_from_center)
        
        elem_x = diff_mag * math.cos(diff_angle) + center[0]
        elem_y = diff_mag * math.sin(diff_angle) + center[1]

        elems.append([int(elem_x), int(elem_y)])
    return elems
#@-node:circ_motion(center, core_elements, rate, rd, td, tick)
#@+node:lin_motion(dir, length, core_elems, rate, tick)
def lin_motion(dir, length, core_elems, rate, tick):
    "differential motion along a line; set length to 0 for unlimited"
    dir_norm = normalize(dir)
    num_elems = len(core_elems)
    
    elems = []
    for x in range(num_elems):
        dist = (float(num_elems - x) / float(num_elems)) * (rate * tick)
        if length > 0 and dist >  length:
            dist = length
        elif length < 0 and (dist*-1) < length:
            dist = dist %  -length
        elem_x = dist * dir_norm[0] + core_elems[x][0]
        elem_y = dist * dir_norm[1] + core_elems[x][1]
        
        elems.append([int(elem_x), int(elem_y)])
    return elems
#@-node:lin_motion(dir, length, core_elems, rate, tick)
#@+node:rad_motion(center, length, core_elements, rate, tick)
def rad_motion(center, length, core_elements, rate, tick):   
    "differential motion along a radius; length >= 0 for outward motion, length < 0 for inward motion"
    num_elems = len(core_elements)
    
    elems = []
    for x in range(num_elems):
        vec_from_center = [core_elements[x][0] - center[0],
                           core_elements[x][1] - center[1]]

        if magnitude(vec_from_center) < 0.05:
            elems.append([int(center[0]), int(center[1])])
            continue

        norm_from_center = normalize(vec_from_center)

        dist = (float(num_elems - x) / float(num_elems)) * (rate * tick)
#        if (length < 0):
#            if (dist - magnitude(vec_from_center)) < 0.05:
#                dist = magnitude(vec_from_center)
#            dist = -dist
#            if (dist < length):
#                dist = length
#        elif (length > 0) and (dist > length):
#            dist = length

        if length < 0:
            dist = -1*dist

        if (length * length) < (dist * dist):
            dist = length
        
        elem_x = dist * norm_from_center[0] + core_elements[x][0]
        elem_y = dist * norm_from_center[1] + core_elements[x][1]

        elems.append([int(elem_x), int(elem_y)])
    return elems

#@-node:rad_motion(center, length, core_elements, rate, tick)
#@+node:oscil_motion(dir, amp, core_elements, rate, tick)
def oscil_motion(dir, amp, core_elems, rate, tick):
    "sinusoidal oscillation perpendicular to dir with amplitude amp"
    dir_norm = normalize(dir)
    num_elems = len(core_elems)
    
    elems = []
    for x in range(num_elems):
        angle = float(num_elems - x) * (rate * tick)
        disp = amp * math.sin(angle)

        elem_x = disp * dir_norm[0] + core_elems[x][0]
        elem_y = disp * dir_norm[1] + core_elems[x][1]
        
        elems.append([int(elem_x), int(elem_y)])
    return elems
#@-node:oscil_motion(dir, amp, core_elements, rate, tick)
#@+node:flat_lin_motion(dir, length, core_elems, rate, tick)
def flat_lin_motion(dir, length, core_elems, rate, tick):
    "uniform motion along a line; set length to 0 for unlimited"
    dir_norm = normalize(dir)
    num_elems = len(core_elems)
    
    elems = []
    for x in range(num_elems):
        dist = (rate * tick)
        if length > 0 and dist >  length:
            dist = length
        elif length < 0 and (dist*-1) < length:
            dist = dist %  -length
        elem_x = dist * dir_norm[0] + core_elems[x][0]
        elem_y = dist * dir_norm[1] + core_elems[x][1]
        
        elems.append([int(elem_x), int(elem_y)])
    return elems
#@-node:flat_lin_motion(dir, length, core_elems, rate, tick)
#@+node:rose_motion(center, core_elements, rate, rd, td, tick)
def rose_motion(center, core_elements, rate, rd, td, tick):   
    "differential motion in a polar coordinate rose pattern"
    num_elems = len(core_elements)
    
    elems = []
    for x in range(num_elems):
        vec_from_center = [core_elements[x][0] - center[0],
                           core_elements[x][1] - center[1]]
        angle_from_center = math.atan2(vec_from_center[1], vec_from_center[0])
        
        diff_angle = (num_elems - x) * (rate * tick) * td
        diff_mag = rd * math.sin(diff_angle)
        
        diff_angle += angle_from_center

        diff_mag += magnitude(vec_from_center)
        
        elem_x = diff_mag * math.cos(diff_angle) + center[0]
        elem_y = diff_mag * math.sin(diff_angle) + center[1]

        elems.append([int(elem_x), int(elem_y)])
    return elems
#@-node:rose_motion(center, core_elements, rate, rd, td, tick)
#@-others
#@-node:@file C:/Projects/PyMotion/elemhandle.py
#@-leo
