#@+leo-ver=4
#@+node:@file C:/Projects/PyMotion/diffmotion.py
#@@language python
#@<< diffmotion declarations >>
#@+node:<< diffmotion declarations >>
#!/usr/bin/env python

import math, pygame
import elemhandle, varticker
from pygame.locals import *

#constants
WINSIZE = [640, 480]
WINCENTER = [320, 240]
NUMSTARS = 150

NUM_FRAMES = 1024
FRAME_RATE = 30


#@-node:<< diffmotion declarations >>
#@nl
#@+others
#@+node:draw_elems
def draw_elems(surface, elements):
    "used to draw (and clear) the elements"
    for elem_set in elements:
        color = elem_set[1]
        for pos in elem_set[0]:
            surface.set_at(pos, color)
            surface.set_at([pos[0],pos[1]+1], color)
            surface.set_at([pos[0],pos[1]-1], color)
            surface.set_at([pos[0]+1,pos[1]], color)
            surface.set_at([pos[0]-1,pos[1]], color)
#@-node:draw_elems
#@+node:main
def main():
    "The main animation code"

    clock = pygame.time.Clock()
    timer = pygame.time.Clock()
    #@    <<set ticker speeds>>
    #@+node:<<set ticker speeds>>
    ticker = varticker.VariableTicker([1.0, 0.8, 60, \
                                       0.5, 0.2, 5, \
    
                                       0.2, 1.0, 30, \
                                       1.0, 2.5, 30, \
                                       2.5, 3.0, 110, \
                                       3.0, 2.5, 10, \
                                       2.5, 3.0, 10, \
                                       3.0, 3.5, 170, \
                                       
                                       3.5, 10.0, 526, \
                                       4.0, 0.01, 10, \
                                       1.0, 0.5, 60, \
                                       1.0, 10.0, 869, \
    
                                       1.0, 2.0, 256, \
                                       1.0, 4.0, 256, \
                                       0.75, 0.75, 128, \
    
                                       1.0, 2.0, 256, \
                                       1.0, 4.0, 256, \
                                       0.75, 0.75, 128, \
                                       
                                       1.0, 0.25, 62, \
                                       0.25, 0.0625, 2, \
                                       0.0625, 0.25, 2, \
                                       0.25, 0.25, 60, \
                                       0.25, 0.0625, 2, \
                                       0.0625, 0.25, 2, \
                                       0.25, 1.0, 126, \
                                       
                                       0.125, 0.5, 64, \
                                       
                                       1.0, 0.25, 60, \
                                       0.25, 0.125, 4, \
                                       0.125, 0.25, 4, \
                                       0.25, 0.25, 56, \
                                       0.25, 0.125, 4, \
                                       0.125, 0.25, 4, \
                                       0.25, 0.25, 56, \
                                       0.25, 0.125, 4, \
                                       0.125, 0.25, 4, \
                                       0.25, 0.25, 56, \
                                       0.25, 0.125, 4, \
    
                                       0.5, 0.25, 4, \
                                       0.25, 8.0, 496, \
                                       8.0, 0.25, 8, \
                                       0.25, 0.0625, 4, \
                                       
                                       0.125, 1.0, 240, \
                                       1.0, 0.5, 80, \
                                       
                                       0.5, 2.0, 250, \
                                       
                                       1.0, 1.0, 65536])
    #@nonl
    #@-node:<<set ticker speeds>>
    #@nl
    
    #create our starfield
    elements = []
    
    #@    <<initialize and prepare screen>>
    #@+node:<<initialize and prepare screen>>
    pygame.init()
    screen = pygame.display.set_mode(WINSIZE)
    pygame.display.set_caption('Differential Motion')
    yellow = 255, 240, 200
    pink = 255, 150, 240
    blue = 120, 150, 255
    black = 20, 20, 40
    screen.fill(black)
    #@nonl
    #@-node:<<initialize and prepare screen>>
    #@nl

    print "Finished initializing screen"
    print "Press any key to start the animation"
    
    #@    <<wait until user starts>>
    #@+node:<<wait until user starts>>
    start = 0
    while not start:
        clock.tick(FRAME_RATE)
        for e in pygame.event.get():
            if e.type == KEYUP:
                start = True
                break
    #@nonl
    #@-node:<<wait until user starts>>
    #@nl

    print "Starting main loop now"
    
    #@    <<DEBUG - advance ticker>>
    #@+node:<<DEBUG - advance ticker>>
    ticker.curr_tick = 0
    #@nonl
    #@-node:<<DEBUG - advance ticker>>
    #@nl

    toggled = 0
        # Just a variable I toggle on and off to output the numbers of the
        # ..first frame in each of the animation segments.

    # main loop
    keepGoing = True
    alphaDone = False
    betaDone = False
    gammaDone = False
    currFrame = 0
    timer.tick()
    while keepGoing:
        
        #@        <<stop if user has pressed a key>>
        #@+node:<<stop if user has pressed a key>>
        for e in pygame.event.get():
            if e.type == KEYUP:
                keepGoing = False
                break
        #@-node:<<stop if user has pressed a key>>
        #@nl
        
        #@        <<erase previous elements>>
        #@+node:<<erase previous elements>>
        # Set all element colors to black for erasure
        for x in range(len(elements)):
            elements[x][1] = black
        draw_elems(screen, elements)
        
        elements = []
        #@nonl
        #@-node:<<erase previous elements>>
        #@nl

        clock.tick(FRAME_RATE)
        
        #@        <<blup alpha motion>>
        #@+node:<<blup alpha motion>>
        if ticker.curr_tick < 65:
            if not toggled:
                print "First extension at frame ", currFrame
                toggled = 1
            my_elements = elemhandle.create_point([200, 320], 8)
            my_elements = elemhandle.lin_motion([1, 0], 64, my_elements, 1, ticker.curr_tick)
            elements.append([my_elements, yellow])
        elif ticker.curr_tick < (360 + 65):
            if toggled:
                print "First rotation at frame ", currFrame
                toggled = 0
            my_elements = elemhandle.create_line([200,320], [1, 0], 74, 8)
            my_elements = elemhandle.circ_motion([200,320], my_elements, math.pi/180, 1, 1, ticker.curr_tick-65)
            elements.append([my_elements, yellow])
        elif ticker.curr_tick < (516 + 425):
            if not toggled:
                print "First retraction at frame ", currFrame
                toggled = 1
            my_elements = elemhandle.create_point([264, 320], 8)
            my_elements = elemhandle.lin_motion([-1, 0], 64, my_elements, 1, ticker.curr_tick-425+68)
            elements.append([my_elements, yellow])
        elif ticker.curr_tick < (10 + 941):
            if toggled:
                print "First erasure at frame ", currFrame
                toggled = 0
            my_elements = [[200,320]]
            elements.append([my_elements, black])
        elif  ticker.curr_tick < 1890:
            # Let blup beta do stuff on its own...
            pass
        elif (ticker.curr_tick < 256 + 1890):
            if toggled == 1:            
                print "expanding motion"
                toggled = 0
            my_elems = elemhandle.create_point([200,320], 8)
            my_elems = elemhandle.lin_motion([1, 0], 64, my_elems, 1, ticker.curr_tick-1890)
            my_elems = elemhandle.circ_motion([200,320], my_elems, 2.0*math.pi/512.0, 1, 1, ticker.curr_tick-1890)
            elements.append([my_elems, yellow])
        elif (ticker.curr_tick < 256 + 2146):
            if toggled == 0:            
                print "joining motion"
                toggled = 1
            my_elems = elemhandle.create_point([264,320], 8)
            my_elems = elemhandle.lin_motion([-1, 0], 128, my_elems, 4, ticker.curr_tick-2146)
            elements.append([my_elems, yellow])
            
            my_elems = elemhandle.create_point([136,320], 8)
            my_elems = elemhandle.lin_motion([1, 0], 128, my_elems, 4, ticker.curr_tick-2146)
            elements.append([my_elems, yellow])
        elif (ticker.curr_tick < 128 + 2402):
            if toggled == 1:
                print "falling"
                toggled = 0
            my_elems = elemhandle.create_point([264,320], 8)
            my_elems = elemhandle.lin_motion([0, 1], 0, my_elems, 16, ticker.curr_tick-2402)
            elements.append([my_elems, yellow])
            
            my_elems = elemhandle.create_point([136,320], 8)
            my_elems = elemhandle.lin_motion([0, 1], 0, my_elems, 16, ticker.curr_tick-2402)
            elements.append([my_elems, yellow])  
        elif (ticker.curr_tick < 3234):
            # let blup beta and gamma have fun
            pass
        elif (ticker.curr_tick < 192 + 3234):
            my_elems = elemhandle.create_point([320, 368], 8)
            my_elems = elemhandle.circ_motion([320, 400], my_elems, 2.0*math.pi/192.0, 1, 1, ticker.curr_tick - 3234)
            elements.append([my_elems, yellow])
            
            my_elems = elemhandle.create_point([320, 112], 8)
            my_elems = elemhandle.circ_motion([320, 80], my_elems, -2.0*math.pi/192.0, 1, 1, ticker.curr_tick - 3234)
            elements.append([my_elems, yellow])
        elif (ticker.curr_tick < 64 + 3426):
            # transition - just spin in place for a while
            my_elems = elemhandle.create_point([320, 368], 8)
            my_elems = elemhandle.circ_motion([320, 400], my_elems, 2.0*math.pi/64.0, 1, 1, ticker.curr_tick - 3426)
            elements.append([my_elems, yellow])
            
            my_elems = elemhandle.create_point([320, 112], 8)
            my_elems = elemhandle.circ_motion([320, 80], my_elems, -2.0*math.pi/64.0, 1, 1, ticker.curr_tick - 3426)
            elements.append([my_elems, yellow])
        elif (ticker.curr_tick < 256 + 3490):
            # get sucked into center, spiralling
            my_elems = elemhandle.create_point([320, 368], 16)
            my_elems = elemhandle.circ_motion([320, 400], my_elems, math.pi/128.0, 1, 1, ticker.curr_tick - 3490)
            my_elems = elemhandle.lin_motion([0, -1], 128, my_elems, 2, ticker.curr_tick - 3490)
            elements.append([my_elems, yellow])
            
            my_elems = elemhandle.create_point([320, 112], 16)
            my_elems = elemhandle.circ_motion([320, 80], my_elems, -math.pi/128.0, 1, 1, ticker.curr_tick - 3490)
            my_elems = elemhandle.lin_motion([0, 1], 128, my_elems, 2, ticker.curr_tick - 3490)
            elements.append([my_elems, yellow])
        else:
            alphaDone = True
        #@nonl
        #@-node:<<blup alpha motion>>
        #@nl
        
        #@        <<blup beta motion>>
        #@+node:<<blup beta motion>>
        if ticker.curr_tick < 951:
            # Let alpha do its work first
            pass
        elif ticker.curr_tick < (53 + 951):
            if not toggled:
                print "Second extension at frame ", currFrame
                toggled = 1
            white = 255, 150, 240
            my_elements = elemhandle.create_point([440+64, 160], 8)
            my_elements = elemhandle.lin_motion([-1,0], 64, my_elements, 1.2, ticker.curr_tick-951)
            elements.append([my_elements, pink])
        elif ticker.curr_tick < (360 + 1004):
            if toggled:
                print "Second rotation at frame ", currFrame
                toggled = 0
            my_elements = elemhandle.create_line([440, 160], [1, 0], 72, 8)
            my_elements = elemhandle.circ_motion([440, 160], my_elements, math.pi/180, 1, -1, ticker.curr_tick-1004)
            elements.append([my_elements, pink])
        elif ticker.curr_tick < (516 + 1364):
            if not toggled:
                print "Second retraction at frame ", currFrame
                toggled = 1
            my_elements = elemhandle.create_point([440+64, 160], 8)
            my_elements = elemhandle.lin_motion([-1,0], 64, my_elements, 1, ticker.curr_tick-1364+68)
            elements.append([my_elements, pink])
        elif ticker.curr_tick < (10 + 1880):
            if toggled:
                print "Second erasure at frame ", currFrame
                toggled = 0
            my_elements = [[440+64,160]]
            elements.append([my_elements, black])
        elif ticker.curr_tick < 2530:
            # let blup alpha do stuff on its own
            pass
        elif (ticker.curr_tick < 256 + 2530):
            if toggled == 0:
                print "expanding motion"
                toggled = 1
            my_elems = elemhandle.create_point([440,160], 8)
            my_elems = elemhandle.lin_motion([-1, 0], 64, my_elems, 1, ticker.curr_tick-2530)
            my_elems = elemhandle.circ_motion([440,160], my_elems, -2.0*math.pi/512.0, 1, 1, ticker.curr_tick-2530)
            elements.append([my_elems, pink])
        elif (ticker.curr_tick < 256 + 2786):
            if toggled == 1:
                print "joining motion"
                toggled = 0
            my_elems = elemhandle.create_point([440+64,160], 8)
            my_elems = elemhandle.lin_motion([-1, 0], 128, my_elems, 4, ticker.curr_tick-2786)
            elements.append([my_elems, pink])
            
            my_elems = elemhandle.create_point([440-64,160], 8)
            my_elems = elemhandle.lin_motion([1, 0], 128, my_elems, 4, ticker.curr_tick-2786)
            elements.append([my_elems, pink])
        elif (ticker.curr_tick < 128 + 3042):
            if toggled == 0:
                print "falling motion"
                toggled = 1
            my_elems = elemhandle.create_point([440+64,160], 8)
            my_elems = elemhandle.lin_motion([0, -1], 159, my_elems, 16, ticker.curr_tick-3042)
            elements.append([my_elems, pink])
            elements.append([[[440+64, 1]], black])
            
            my_elems = elemhandle.create_point([440-64,160], 8)
            my_elems = elemhandle.lin_motion([0, -1], 159, my_elems, 16, ticker.curr_tick-3042)
            elements.append([my_elems, pink])
            elements.append([[[440-64, 1]], black])
        elif (ticker.curr_tick < 3298):
            # let blup gamma have fun
            pass
        elif (ticker.curr_tick < 128 + 3298):
            # little circling thingies
            my_elems = elemhandle.create_line([192, 240], [-1, 0], 64, 8)
            my_elems.reverse()
            my_elems = elemhandle.circ_motion([128, 240], my_elems, -2.0*math.pi/128.0, 1, 1, ticker.curr_tick - 3298)
            elements.append([my_elems, pink])
            
            my_elems = elemhandle.create_line([448, 240], [1, 0], 64, 8)
            my_elems.reverse()
            my_elems = elemhandle.circ_motion([512, 240], my_elems, 2.0*math.pi/128.0, 1, 1, ticker.curr_tick - 3298)
            elements.append([my_elems, pink])
        elif (ticker.curr_tick < 64 + 3426):
            # transition - this should disappear
            my_elems = elemhandle.create_point([192, 240], 8)
            my_elems = elemhandle.circ_motion([192-64, 240], my_elems, -2.0*math.pi/64.0, 1, 1, ticker.curr_tick - 3426)
            my_elems = elemhandle.rad_motion([192-64, 240], -64, my_elems, 1, ticker.curr_tick - 3426)
            elements.append([my_elems, pink])
            
            my_elems = elemhandle.create_point([448, 240], 8)
            my_elems = elemhandle.circ_motion([512, 240], my_elems, -2.0*math.pi/64.0, 1, 1, ticker.curr_tick - 3426)
            my_elems = elemhandle.rad_motion([512, 240], -64, my_elems, 1, ticker.curr_tick - 3426)
            elements.append([my_elems, pink])
        else:
            betaDone = True
        #@nonl
        #@-node:<<blup beta motion>>
        #@nl
        
        #@        <<blup gamma  motion>>
        #@+node:<<blup gamma  motion>>
        if ticker.curr_tick < 3170:
            # let alpha and beta do their thing
            pass
        elif (ticker.curr_tick < 256 + 3170):
            if toggled == 1:            
                print "expanding motion"
                toggled = 0
            my_elems = elemhandle.create_point([320,240], 128)
            my_elems = elemhandle.lin_motion([1, 0], 64, my_elems, 1, ticker.curr_tick-3170)
            my_elems = elemhandle.rad_motion([320,240], 64, my_elems, 1, ticker.curr_tick-3170)
            my_elems = elemhandle.circ_motion([320,240], my_elems, 2.0*math.pi/512.0, 1, 1, ticker.curr_tick-3170)
            elements.append([my_elems, blue])
        elif (ticker.curr_tick < 64 + 3426):
            # transitional stuff
            my_elems = elemhandle.create_line([320, 240], [0, 1], 128, 64)
            my_elems = elemhandle.circ_motion([320, 240], my_elems, 2.0*math.pi/64.0, 1, 1, ticker.curr_tick-3426)
            elements.append([my_elems, blue])
        elif (ticker.curr_tick < 256 + 3490):
            if toggled == 1:            
                print "happy spiral motion"
                toggled = 0
            my_elems = elemhandle.create_point([320,240], 128)
            my_elems = elemhandle.lin_motion([1, 0], -128, my_elems, 1, ticker.curr_tick-3490)
            my_elems = elemhandle.circ_motion([320,240], my_elems, -2.0*math.pi/512.0, 1, 1, ticker.curr_tick-3490)
            elements.append([my_elems, blue])
        elif (ticker.curr_tick < 512 + 3746):
            # cardioid-looking contraction
            
            my_elems = elemhandle.create_point([320+128,240], 128)
            my_elems = elemhandle.rad_motion([320, 240], -128, my_elems, 32, ticker.curr_tick-3746)
            my_elems_a = []
            my_elems_b = []
            
            for curr_elem in range(len(my_elems)):
                if (curr_elem % 2 == 0):
                    my_elems_a.append(my_elems[curr_elem])
                else:
                    my_elems_b.append(my_elems[curr_elem])
        
            my_elems_a = elemhandle.circ_motion([320,240], my_elems_a, 2.0*math.pi/256.0, 1, 1, ticker.curr_tick-3746)
            my_elems_b = elemhandle.circ_motion([320,240], my_elems_b, -2.0*math.pi/256.0, 1, 1, ticker.curr_tick-3746)
        
            my_elems = my_elems_a
            my_elems.extend(my_elems_b)
            elements.append([my_elems, blue])
        elif (ticker.curr_tick < 320 + 4258):
            # double cone expansion
            
            my_elems = elemhandle.create_line([320, 240], [1, 0], 128, 64)
            my_elems.reverse()
            my_elems = elemhandle.circ_motion([320, 240], my_elems, -2.0*math.pi/320.0, 1, 1, ticker.curr_tick-4258)
            my_elems = elemhandle.lin_motion([1, 0], 0, my_elems, 2, ticker.curr_tick - 4258)
            elements.append([my_elems, blue])
            
            my_elems = elemhandle.create_line([320, 240], [-1, 0], 128, 64)
            my_elems.reverse()
            my_elems = elemhandle.circ_motion([320, 240], my_elems, 2.0*math.pi/320.0, 1, 1, ticker.curr_tick-4258)
            my_elems = elemhandle.lin_motion([-1, 0], 0, my_elems, 2, ticker.curr_tick - 4258)
            elements.append([my_elems, blue])
        elif (ticker.curr_tick < 250 + 4578):
            # "snowing" motion
            
            my_elems = elemhandle.create_line([40, 240], [1, 0], 600, 8)
            
            for curr_elem in range(len(my_elems)):
                my_points = elemhandle.create_point(my_elems[curr_elem], 16)
                my_points = elemhandle.lin_motion([0, 1], 0, my_points, 16, ticker.curr_tick - 4578)
                
                if curr_elem % 2 == 0:
                    my_points = elemhandle.oscil_motion([0, 1], 16, my_points, 0.125, ticker.curr_tick - 4578)
                    my_points = elemhandle.oscil_motion([1, 0], 8, my_points, 0.125, ticker.curr_tick - 4578)
                else:
                    my_points = elemhandle.oscil_motion([0, -1], 16, my_points, 0.125, ticker.curr_tick - 4578)
                    my_points = elemhandle.oscil_motion([1, 0], 8, my_points, 0.125, ticker.curr_tick - 4578)
                elements.append([my_points, blue])
        else:
            gammaDone = True
        #@nonl
        #@-node:<<blup gamma  motion>>
        #@nl
        
        #@        <<output ending stats and break loop>>
        #@+node:<<output ending stats and break loop>>
        if alphaDone and betaDone and gammaDone:
            print "Final tick: ", ticker.curr_tick
            print "Final frame: ", currFrame
            keepGoing = False
        #@nonl
        #@-node:<<output ending stats and break loop>>
        #@nl

        draw_elems(screen, elements)
        pygame.display.update()
        
        ticker.tick()
        
        currFrame += 1


    timeElapsed = timer.tick()
    timeElapsed = float(timeElapsed) / 1000.0

    print "Finished with main loop"
    print "Seconds elapsed: ", timeElapsed

    # Blank ticker for next time
    ticker = 0
    
    #@    <<wait until user presses a key to quit>>
    #@+node:<<wait until user presses a key to quit>>
    start = 0
    while not start:
        clock.tick(FRAME_RATE)
        for e in pygame.event.get():
            if e.type == KEYUP:
                start = True
                break
    #@nonl
    #@-node:<<wait until user presses a key to quit>>
    #@nl
#@+node:Initialization
#@-node:Initialization
#@+node:Main Loop
#@-node:Main Loop
#@+node:Cleanup
#@-node:Cleanup
#@-node:main
#@-others


# if python says run, then we should run
if __name__ == '__main__':
	main()


#@-node:@file C:/Projects/PyMotion/diffmotion.py
#@-leo
